##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Search interface for queriables.

"""
__docformat__ = "reStructuredText"

from zope.formlib.interfaces import IInputWidget
from zope.formlib.interfaces import InputErrors
from zope.formlib.interfaces import ISourceQueryView
from zope.formlib.interfaces import MissingInputError
from zope.formlib.interfaces import WidgetsError
from zope.formlib.utility import setUpWidgets
from zope.i18n import translate
from zope.interface import implementer
from zope.schema import getFieldsInOrder
from zope.traversing.api import getName
from zope.traversing.api import getPath

from zope.app.authentication.i18n import ZopeMessageFactory as _


search_label = _('search-button', 'Search')
source_label = _("Source path")
source_title = _("Path to the source utility")


@implementer(ISourceQueryView)
class QuerySchemaSearchView:

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def render(self, name):
        schema = self.context.schema
        sourcename = getName(self.context)
        sourcepath = getPath(self.context)
        setUpWidgets(self, schema, IInputWidget, prefix=name + '.field')
        html = []

        # add sub title for source search field
        html.append('<h4>%s</h4>' % sourcename)

        # start row for path display field
        html.append('<div class="row">')

        # for each source add path of source
        html.append('  <div class="label">')
        label = translate(source_label, context=self.request)
        title = translate(source_title, context=self.request)
        html.append(f'    <label for="{sourcename}" title="{title}">')
        html.append('      %s' % label)
        html.append('    </label>')
        html.append('  </div>')
        html.append('  <div class="field">')
        html.append('      %s' % sourcepath)
        html.append('  </div>')
        html.append('</div>')

        # start row for search fields
        html.append('<div class="row">')

        for field_name, _field in getFieldsInOrder(schema):
            widget = getattr(self, field_name + '_widget')

            # for each field add label...
            html.append('  <div class="label">')
            html.append('    <label for="%s" title="%s">'
                        % (widget.name, widget.hint))
            html.append('      %s' % widget.label)
            html.append('    </label>')
            html.append('  </div>')

            # ...and field widget
            html.append('  <div class="field">')
            html.append('    %s' % widget())

            if widget.error():  # pragma: no cover
                html.append('    <div class="error">')
                html.append('      %s' % widget.error())
                html.append('    </div>')
            html.append('  </div>')
        # end row
        html.append('</div>')

        # add search button for search fields
        html.append('<div class="row">')
        html.append('  <div class="field">')
        html.append('    <input type="submit" name="%s" value="%s" />'
                    % (name + '.search',
                       translate(search_label, context=self.request)))
        html.append('  </div>')
        html.append('</div>')

        return '\n'.join(html)

    def results(self, name):
        if (name + '.search') not in self.request:
            return None
        schema = self.context.schema
        setUpWidgets(self, schema, IInputWidget, prefix=name + '.field')
        # XXX inline the original getWidgetsData call in
        # zope.app.form.utility to lift the dependency on zope.app.form.
        data = {}
        errors = []
        for widget_name, field in getFieldsInOrder(schema):
            widget = getattr(self, widget_name + '_widget')
            if IInputWidget.providedBy(widget):
                if widget.hasInput():
                    try:
                        data[widget_name] = widget.getInputValue()
                    except InputErrors as error:  # pragma: no cover
                        errors.append(error)
                elif field.required:  # pragma: no cover
                    errors.append(MissingInputError(
                        widget_name, widget.label, 'the field is required'))
        if errors:  # pragma: no cover
            raise WidgetsError(errors, widgetsData=data)
        return self.context.search(data)



##############################################################################
#
# Copyright (c) 2017 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"Test helpers; not public API"

from zope.interface import Interface
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from zope import component


def provideUtility(provided, utility, name=''):
    gsm = component.getGlobalSiteManager()
    gsm.registerUtility(utility, provided, name, event=False)


stypes = list, tuple


def provideAdapter(required, provided, factory, name='', using=None, **kw):
    assert 'with' not in kw
    assert not isinstance(factory, stypes), "Factory cannot be a list or tuple"

    gsm = component.getGlobalSiteManager()

    if using:
        required = (required, ) + tuple(using)
    elif not isinstance(required, stypes):
        required = (required,)

    gsm.registerAdapter(factory, required, provided, name, event=False)


def browserView(for_, name, factory, layer=IDefaultBrowserLayer,
                providing=Interface):
    """Define a global browser view
    """
    provideAdapter(for_, providing, factory, name, (layer,))


def browserViewProviding(for_, factory, providing, layer=IDefaultBrowserLayer):
    """Define a view providing a particular interface."""
    return browserView(for_, '', factory, layer, providing)


def provideMultiView(for_, type, providing, name, factory, layer=None):
    if layer is None:
        layer = type
    provideAdapter(for_[0], providing, factory,
                   name, tuple(for_[1:]) + (layer,))



##############################################################################
#
# Copyright (c) 2009 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Login Form

"""
from zope.authentication.interfaces import IUnauthenticatedPrincipal


class LoginForm:
    """Mix-in class to implement login form logic"""

    context = None
    request = None
    unauthenticated = None
    camefrom = None

    def __call__(self):
        request = self.request
        principal = request.principal

        unauthenticated = IUnauthenticatedPrincipal.providedBy(principal)
        self.unauthenticated = unauthenticated

        camefrom = request.get('camefrom')
        if isinstance(camefrom, list):
            # Beginning on python2.6 this happens if the parameter is
            # supplied more than once
            camefrom = camefrom[0]
        self.camefrom = camefrom

        if not unauthenticated and 'SUBMIT' in request:
            # authenticated by submitting
            request.response.redirect(camefrom or '.')
            return ''

        return self.index()  # call template



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Pluggable Authentication Utility Interfaces"""

import zope.interface
import zope.schema
import zope.security.interfaces
# BBB: the password managers were moved into zope.password package.
from zope.password.interfaces import IPasswordManager
# BBB: using zope.pluggableauth
from zope.pluggableauth.interfaces import AuthenticatedPrincipalCreated
from zope.pluggableauth.interfaces import FoundPrincipalCreated
from zope.pluggableauth.interfaces import IAuthenticatedPrincipalCreated
from zope.pluggableauth.interfaces import IAuthenticatedPrincipalFactory
from zope.pluggableauth.interfaces import IAuthenticatorPlugin
from zope.pluggableauth.interfaces import ICredentialsPlugin
from zope.pluggableauth.interfaces import IFoundPrincipalCreated
from zope.pluggableauth.interfaces import IFoundPrincipalFactory
from zope.pluggableauth.interfaces import IGroupAdded
from zope.pluggableauth.interfaces import IPluggableAuthentication
from zope.pluggableauth.interfaces import IPlugin
from zope.pluggableauth.interfaces import IPrincipal
from zope.pluggableauth.interfaces import IPrincipalCreated
from zope.pluggableauth.interfaces import IPrincipalFactory
from zope.pluggableauth.interfaces import IPrincipalInfo
from zope.pluggableauth.interfaces import IPrincipalsAddedToGroup
from zope.pluggableauth.interfaces import IPrincipalsRemovedFromGroup
from zope.pluggableauth.interfaces import IQueriableAuthenticator
from zope.pluggableauth.interfaces import IQuerySchemaSearch
# BBB: using zope.pluggableauth
from zope.pluggableauth.plugins.groupfolder import GroupAdded

from zope.app.authentication.i18n import ZopeMessageFactory as _



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Test IRolePermissionManager class that has no context.
"""

from zope.interface import implementer
from zope.securitypolicy.interfaces import Allow
from zope.securitypolicy.interfaces import Deny
from zope.securitypolicy.interfaces import IRolePermissionManager
from zope.securitypolicy.interfaces import IRolePermissionMap
from zope.securitypolicy.securitymap import SecurityMap


@implementer(IRolePermissionManager, IRolePermissionMap)
class RolePermissionManager:
    """
    provide adapter that manages role permission data in an object attribute
    """

    def __init__(self):
        self._rp = SecurityMap()

    def grantPermissionToRole(self, permission_id, role_id):
        ''' See the interface IRolePermissionManager '''
        rp = self._getRolePermissions(create=1)
        rp.addCell(permission_id, role_id, Allow)

    def denyPermissionToRole(self, permission_id, role_id):
        ''' See the interface IRolePermissionManager '''
        rp = self._getRolePermissions(create=1)
        rp.addCell(permission_id, role_id, Deny)

    def unsetPermissionFromRole(self, permission_id, role_id):
        ''' See the interface IRolePermissionManager '''
        rp = self._getRolePermissions()
        # Only unset if there is a security map, otherwise, we're done
        if rp:
            rp.delCell(permission_id, role_id)

    def getRolesForPermission(self, permission_id):
        '''See interface IRolePermissionMap'''
        rp = self._getRolePermissions()
        if rp:
            return rp.getRow(permission_id)
        raise NotImplementedError("Should never get here")

    def getPermissionsForRole(self, role_id):
        '''See interface IRolePermissionMap'''
        rp = self._getRolePermissions()
        if rp:
            return rp.getCol(role_id)
        raise NotImplementedError("Should never get here")

    def getRolesAndPermissions(self):
        '''See interface IRolePermissionMap'''
        raise NotImplementedError("Not used by tests")

    def getSetting(self, permission_id, role_id):
        '''See interface IRolePermissionMap'''
        raise NotImplementedError("Not used by tests")

    def _getRolePermissions(self, create=0):
        """Get the role permission map stored in the context, optionally
           creating one if necessary"""
        return self._rp



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Backward compatibility imports for password managers."""

# BBB: the password managers were moved into zope.password package.
from zope.password.password import MD5PasswordManager
from zope.password.password import PlainTextPasswordManager
from zope.password.password import SHA1PasswordManager
from zope.password.password import SSHAPasswordManager
from zope.password.password import managers
from zope.password.vocabulary import PasswordManagerNamesVocabulary



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Zope Groups Folder implementation."""

# BBB using zope.pluggableauth.plugin.groupfolder
from zope.pluggableauth.plugins.groupfolder import GroupCycle
from zope.pluggableauth.plugins.groupfolder import GroupFolder
from zope.pluggableauth.plugins.groupfolder import GroupInfo
from zope.pluggableauth.plugins.groupfolder import GroupInformation
from zope.pluggableauth.plugins.groupfolder import IGroupContained
from zope.pluggableauth.plugins.groupfolder import IGroupFolder
from zope.pluggableauth.plugins.groupfolder import IGroupInformation
from zope.pluggableauth.plugins.groupfolder import IGroupPrincipalInfo
from zope.pluggableauth.plugins.groupfolder import IGroupSearchCriteria
from zope.pluggableauth.plugins.groupfolder import InvalidGroupId
from zope.pluggableauth.plugins.groupfolder import InvalidPrincipalIds
from zope.pluggableauth.plugins.groupfolder import nocycles
from zope.pluggableauth.plugins.groupfolder import setGroupsForPrincipal
from zope.pluggableauth.plugins.groupfolder import setMemberSubscriber
from zope.pluggableauth.plugins.groupfolder import specialGroups



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Role Permission View Classes

"""
from datetime import datetime

from zope.component import getUtilitiesFor
from zope.component import getUtility
from zope.exceptions.interfaces import UserError
from zope.i18n import translate
from zope.i18nmessageid import ZopeMessageFactory as _
from zope.interface import implementer
from zope.security.interfaces import IPermission
from zope.securitypolicy.interfaces import Allow
from zope.securitypolicy.interfaces import Deny
from zope.securitypolicy.interfaces import IRole
from zope.securitypolicy.interfaces import IRolePermissionManager
from zope.securitypolicy.interfaces import Unset


class RolePermissionView:

    _pagetip = _("""For each permission you want to grant (or deny) to a role,
        set the entry for that permission and role to a '+' (or '-').
        Permissions are shown on the left side, going down.
        Roles are shown accross the top.
        """)

    request = None
    context = None
    _roles = None
    _permissions = None

    def pagetip(self):
        return translate(self._pagetip, context=self.request)

    def roles(self):
        roles = getattr(self, '_roles', None)
        if roles is None:
            roles = [
                (translate(role.title, context=self.request).strip(), role)
                for name, role in getUtilitiesFor(IRole)]
            roles.sort()
            roles = self._roles = [role for name, role in roles]
        return roles

    def permissions(self):
        permissions = getattr(self, '_permissions', None)
        if permissions is None:
            permissions = [
                (translate(perm.title, context=self.request).strip(), perm)
                for name, perm in getUtilitiesFor(IPermission)
                if name != 'zope.Public']
            permissions.sort()
            permissions = self._permissions = [perm
                                               for name, perm in permissions]

        return permissions

    def availableSettings(self, noacquire=False):
        aq = {'id': Unset.getName(), 'shorttitle': ' ',
              'title': _('permission-acquire', 'Acquire')}
        rest = [{'id': Allow.getName(), 'shorttitle': '+',
                 'title': _('permission-allow', 'Allow')},
                {'id': Deny.getName(), 'shorttitle': '-',
                 'title': _('permission-deny', 'Deny')},
                ]

        return rest if noacquire else [aq] + rest

    def permissionRoles(self):
        context = self.context.__parent__
        roles = self.roles()
        return [PermissionRoles(permission, context, roles)
                for permission in self.permissions()]

    def permissionForID(self, pid):
        roles = self.roles()
        perm = getUtility(IPermission, pid)
        return PermissionRoles(perm, self.context.__parent__, roles)

    def roleForID(self, rid):
        permissions = self.permissions()
        role = getUtility(IRole, rid)
        return RolePermissions(role, self.context.__parent__, permissions)

    def update(self, testing=None):
        status = ''
        changed = False

        if 'SUBMIT' in self.request:
            roles = [r.id for r in self.roles()]
            permissions = [p.id for p in self.permissions()]
            prm = IRolePermissionManager(self.context.__parent__)
            for ip in range(len(permissions)):
                rperm = self.request.get("p%s" % ip)
                if rperm not in permissions:
                    continue
                for ir in range(len(roles)):
                    rrole = self.request.get("r%s" % ir)
                    if rrole not in roles:
                        continue
                    setting = self.request.get(f"p{ip}r{ir}", None)
                    if setting is not None:
                        if setting == Unset.getName():
                            prm.unsetPermissionFromRole(rperm, rrole)
                        elif setting == Allow.getName():
                            prm.grantPermissionToRole(rperm, rrole)
                        elif setting == Deny.getName():
                            prm.denyPermissionToRole(rperm, rrole)
                        else:
                            raise ValueError("Incorrect setting: %s"
                                             % setting)  # pragma: no cover
            changed = True

        if 'SUBMIT_PERMS' in self.request:
            prm = IRolePermissionManager(self.context.__parent__)
            roles = self.roles()
            rperm = self.request.get('permission_id')
            settings = self.request.get('settings', ())
            for ir in range(len(roles)):
                rrole = roles[ir].id
                setting = settings[ir]
                if setting == Unset.getName():
                    prm.unsetPermissionFromRole(rperm, rrole)
                elif setting == Allow.getName():
                    prm.grantPermissionToRole(rperm, rrole)
                elif setting == Deny.getName():
                    prm.denyPermissionToRole(rperm, rrole)
                else:
                    raise ValueError("Incorrect setting: %s" % setting)
            changed = True

        if 'SUBMIT_ROLE' in self.request:
            role_id = self.request.get('role_id')
            prm = IRolePermissionManager(self.context.__parent__)
            allowed = self.request.get(Allow.getName(), ())
            denied = self.request.get(Deny.getName(), ())
            for permission in self.permissions():
                rperm = permission.id
                if rperm in allowed and rperm in denied:
                    permission_translated = translate(
                        permission.title, context=self.request)
                    msg = _('You choose both allow and deny for permission'
                            ' "${permission}". This is not allowed.',
                            mapping={'permission': permission_translated})
                    raise UserError(msg)
                if rperm in allowed:
                    prm.grantPermissionToRole(rperm, role_id)
                elif rperm in denied:
                    prm.denyPermissionToRole(rperm, role_id)
                else:
                    prm.unsetPermissionFromRole(rperm, role_id)
            changed = True

        if changed:
            formatter = self.request.locale.dates.getFormatter(
                'dateTime', 'medium')
            status = _("Settings changed at ${date_time}",
                       mapping={'date_time':
                                formatter.format(datetime.utcnow())})

        return status


@implementer(IPermission)
class PermissionRoles:

    def __init__(self, permission, context, roles):
        self._permission = permission
        self._context = context
        self._roles = roles

    @property
    def id(self):
        return self._permission.id

    @property
    def title(self):
        return self._permission.title

    @property
    def description(self):
        return self._permission.description

    def roleSettings(self):
        """
        Returns the list of setting names of each role for this permission.
        """
        prm = IRolePermissionManager(self._context)
        proles = prm.getRolesForPermission(self._permission.id)
        settings = {}
        for role, setting in proles:
            settings[role] = setting.getName()
        nosetting = Unset.getName()
        return [settings.get(role.id, nosetting) for role in self._roles]


@implementer(IRole)
class RolePermissions:

    def __init__(self, role, context, permissions):
        self._role = role
        self._context = context
        self._permissions = permissions

    @property
    def id(self):
        return self._role.id

    @property
    def title(self):
        return self._role.title

    @property
    def description(self):
        return self._role.description

    def permissionsInfo(self):
        prm = IRolePermissionManager(self._context)
        rperms = prm.getPermissionsForRole(self._role.id)
        settings = {}
        for permission, setting in rperms:
            settings[permission] = setting.getName()
        nosetting = Unset.getName()
        return [{'id': permission.id,
                 'title': permission.title,
                 'setting': settings.get(permission.id, nosetting)}
                for permission in self._permissions]



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""ZODB-based Authentication Source"""

from zope.pluggableauth.factories import AuthenticatedPrincipalFactory
from zope.pluggableauth.factories import FoundPrincipalFactory
from zope.pluggableauth.factories import Principal
from zope.pluggableauth.factories import PrincipalInfo
# BBB using zope.pluggableauth
from zope.pluggableauth.interfaces import IAuthenticatorPlugin
# BBB using zope.pluggableauth.plugins.principalfolder
from zope.pluggableauth.plugins.principalfolder import IInternalPrincipal
from zope.pluggableauth.plugins.principalfolder import \
    IInternalPrincipalContained
from zope.pluggableauth.plugins.principalfolder import \
    IInternalPrincipalContainer
from zope.pluggableauth.plugins.principalfolder import InternalPrincipal
from zope.pluggableauth.plugins.principalfolder import ISearchSchema
from zope.pluggableauth.plugins.principalfolder import PrincipalFolder



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Pluggable Authentication Service Tests
"""
import doctest
import re
import unittest

from zope.annotation.attribute import AttributeAnnotations
from zope.authentication.interfaces import IAuthentication
from zope.browsermenu.menu import getFirstMenuItem
from zope.component import getUtility
from zope.component import provideAdapter
from zope.component import provideHandler
from zope.component import provideUtility
from zope.component.eventtesting import PlacelessSetup as EventPlacelessSetup
from zope.component.eventtesting import clearEvents
from zope.component.eventtesting import getEvents
from zope.component.hooks import resetHooks
from zope.component.hooks import setHooks
from zope.component.hooks import setSite
from zope.component.interfaces import ISite
from zope.component.testing import tearDown
from zope.container.interfaces import ISimpleReadContainer
from zope.container.traversal import ContainerTraversable
from zope.interface import Interface
from zope.interface import implementer
from zope.interface.interfaces import IComponentLookup
from zope.password.testing import setUpPasswordManagers
from zope.pluggableauth.plugins.session import SessionCredentialsPlugin
from zope.publisher import base
from zope.publisher.browser import BrowserPage
from zope.publisher.browser import BrowserView
from zope.publisher.interfaces import IRequest
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.session.http import CookieClientIdManager
from zope.session.interfaces import IClientId
from zope.session.interfaces import IClientIdManager
from zope.session.interfaces import ISession
from zope.session.interfaces import ISessionDataContainer
from zope.session.session import ClientId
from zope.session.session import PersistentSessionDataContainer
from zope.session.session import Session
from zope.site.folder import rootFolder
from zope.site.site import LocalSiteManager
from zope.site.site import SiteManagerAdapter
from zope.testing import renormalizing
from zope.testing.cleanup import CleanUp
from zope.traversing import api as traversing
from zope.traversing.interfaces import ITraversable

from zope import component


def setUpTraversal():
    from zope.traversing.testing import setUp as tSetUp
    tSetUp()
    component.provideAdapter(ContainerTraversable,
                             (ISimpleReadContainer,),
                             ITraversable)


def setUpAnnotations():
    component.provideAdapter(AttributeAnnotations)


def setUpSiteManagerLookup():
    component.provideAdapter(SiteManagerAdapter,
                             (Interface,),
                             IComponentLookup)


def createSiteManager(folder, setsite=False):
    if not ISite.providedBy(folder):
        folder.setSiteManager(LocalSiteManager(folder))
    if setsite:
        setSite(folder)
    return traversing.traverse(folder, "++etc++site")


class PlacelessSetup(EventPlacelessSetup,
                     CleanUp):

    def setUp(self, doctesttest=None):
        EventPlacelessSetup.setUp(self)

        setUpPasswordManagers()


setUp = PlacelessSetup().setUp


def placefulSetUp(site=False):
    PlacelessSetup().setUp()
    setHooks()
    setUpTraversal()

    if site:
        site = rootFolder()
        createSiteManager(site, setsite=True)
        return site


def placefulTearDown():
    PlacelessSetup().tearDown()
    resetHooks()
    setSite()


def siteSetUp(test):
    placefulSetUp(site=True)


def siteTearDown(test):
    placefulTearDown()


def nonHTTPSessionTestCaseSetUp(sdc_class=PersistentSessionDataContainer):
    # I am getting an error with ClientId and not TestClientId
    setUp()
    provideAdapter(ClientId, [IRequest], IClientId)
    provideAdapter(Session, [IRequest], ISession)
    provideUtility(CookieClientIdManager(), IClientIdManager)
    sdc = sdc_class()
    provideUtility(sdc, ISessionDataContainer, '')


class NonHTTPSessionTestCase(unittest.TestCase):
    # Small test suite to catch an error with non HTTP protocols, like FTP
    # and SessionCredentialsPlugin.
    def setUp(self):
        nonHTTPSessionTestCaseSetUp()

    def tearDown(self):
        placefulTearDown()

    def test_exeractCredentials(self):
        plugin = SessionCredentialsPlugin()

        self.assertEqual(plugin.extractCredentials(
            base.TestRequest('/')), None)

    def test_challenge(self):
        plugin = SessionCredentialsPlugin()

        self.assertEqual(plugin.challenge(base.TestRequest('/')), False)

    def test_logout(self):
        plugin = SessionCredentialsPlugin()

        self.assertEqual(plugin.logout(base.TestRequest('/')), False)


class TestPlacesssetup(unittest.TestCase):

    def test_runs(self):
        from zope.app.authentication import placelesssetup
        placelesssetup.PlacelessSetup().setUp()


@implementer(IBrowserPublisher)
class ManagementViewSelector(BrowserView):
    """View that selects the first available management view.

    Support 'zmi_views' actions like: 'javascript:alert("hello")',
    '../view_on_parent.html' or '++rollover++'.
    """
    # Copied from zope.app.publication
    # Simplified to assert just the test case we expect.

    def browserDefault(self, request):
        return self, ()

    def __call__(self):
        item = getFirstMenuItem('zmi_views', self.context, self.request)
        assert item
        redirect_url = item['action']
        if not redirect_url.lower().startswith(('../', 'javascript:', '++')):
            self.request.response.redirect(redirect_url)
            return ''
        raise AssertionError("Should not get here")  # pragma: no cover


class Unauthorized(BrowserPage):

    def template(self):
        return "You are not authorized"

    def __call__(self):
        # Set the error status to 403 (Forbidden) in the case when we don't
        # challenge the user
        self.request.response.setStatus(403)

        # make sure that squid does not keep the response in the cache
        self.request.response.setHeader(
            'Expires', 'Mon, 26 Jul 1997 05:00:00 GMT')
        self.request.response.setHeader(
            'Cache-Control', 'no-store, no-cache, must-revalidate')
        self.request.response.setHeader('Pragma', 'no-cache')

        principal = self.request.principal
        auth = getUtility(IAuthentication)
        auth.unauthorized(principal.id, self.request)
        if self.request.response.getStatus() not in (302, 303):
            return self.template()


checker = renormalizing.RENormalizing([
    (re.compile("u('[^']*?')"), r"\1"),
    (re.compile('u("[^"]*?")'), r"\1"),
])


def test_suite():
    flags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS
    return unittest.TestSuite((
        doctest.DocTestSuite('zope.app.authentication.interfaces'),
        doctest.DocTestSuite('zope.app.authentication.password'),
        doctest.DocTestSuite('zope.app.authentication.generic'),
        doctest.DocTestSuite('zope.app.authentication.httpplugins'),
        doctest.DocTestSuite('zope.app.authentication.ftpplugins'),
        doctest.DocTestSuite('zope.app.authentication.groupfolder'),
        doctest.DocFileSuite('principalfolder.rst',
                             optionflags=flags,
                             checker=checker,
                             setUp=setUp,
                             tearDown=tearDown),
        doctest.DocTestSuite('zope.app.authentication.principalfolder',
                             optionflags=flags,
                             checker=checker,
                             setUp=setUp,
                             tearDown=tearDown),
        doctest.DocTestSuite('zope.app.authentication.idpicker'),
        doctest.DocTestSuite('zope.app.authentication.session',
                             optionflags=flags,
                             checker=checker,
                             setUp=siteSetUp,
                             tearDown=siteTearDown),
        doctest.DocFileSuite('README.rst',
                             setUp=siteSetUp,
                             tearDown=siteTearDown,
                             optionflags=flags,
                             checker=checker,
                             globs={'provideUtility': provideUtility,
                                    'provideAdapter': provideAdapter,
                                    'provideHandler': provideHandler,
                                    'getEvents': getEvents,
                                    'clearEvents': clearEvents,
                                    }),
        doctest.DocFileSuite('groupfolder.rst',
                             setUp=setUp,
                             tearDown=tearDown,
                             optionflags=flags,
                             checker=checker,
                             ),
        doctest.DocFileSuite('vocabulary.rst',
                             setUp=setUp,
                             tearDown=tearDown,
                             optionflags=flags,
                             checker=checker,
                             ),
        unittest.defaultTestLoader.loadTestsFromName(__name__)
    ))


if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Granting Roles and Permissions to Principals

"""

__docformat__ = "reStructuredText"

import base64

import zope.schema
from zope.authentication.principal import PrincipalSource
from zope.component import getUtilitiesFor
from zope.formlib.interfaces import IInputWidget
from zope.formlib.interfaces import MissingInputError
from zope.formlib.utility import setUpWidget
from zope.formlib.widget import renderElement
from zope.formlib.widgets import RadioWidget
from zope.i18nmessageid import ZopeMessageFactory as _
from zope.schema.vocabulary import SimpleTerm
from zope.security.interfaces import IPermission
from zope.securitypolicy.interfaces import Allow
from zope.securitypolicy.interfaces import Deny
from zope.securitypolicy.interfaces import IPrincipalPermissionManager
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.securitypolicy.interfaces import IRole
from zope.securitypolicy.interfaces import Unset
from zope.securitypolicy.vocabulary import GrantVocabulary


try:
    text_type = unicode
except NameError:
    text_type = str

settings_vocabulary = GrantVocabulary([
    SimpleTerm(Allow, token="allow", title=_('Allow')),
    SimpleTerm(Unset, token="unset", title=_('Unset')),
    SimpleTerm(Deny, token='deny', title=_('Deny')),
])


class GrantWidget(RadioWidget):
    """Grant widget for building a colorized matrix.

    The matrix shows anytime the status if you edit the radio widgets.
    This special widget shows the radio input field without labels.
    The labels are added in the header of the table. The order of the radio
    input fields is 'Allowed', 'Unset', 'Deny'.

    """
    orientation = "horizontal"
    _tdTemplate = (
        '\n<td class="%s">\n<center>\n<label for="%s" title="%s">\n'
        '%s\n</label>\n</center>\n</td>\n'
    )

    def __call__(self):
        """See IBrowserWidget."""
        value = self._getFormValue()
        return self.renderValue(value)

    def renderItem(self, index, text, value, name, cssClass):
        """Render an item of the list.

        Revert the order of label and text. Added field id to the lable
        attribute.

        Added tabel td tags for fit in the matrix table.

        """

        tdClass = ''
        id = f'{name}.{index}'
        elem = renderElement('input',
                             value=value,
                             name=name,
                             id=id,
                             cssClass=cssClass,
                             type='radio',
                             extra='onclick="changeMatrix(this);"')
        return self._tdTemplate % (tdClass, id, text, elem)

    def renderSelectedItem(self, index, text, value, name, cssClass):
        """Render a selected item of the list.

        Revert the order of label and text. Added field id to the lable
        attribute.
        """

        tdClass = 'default'
        id = f'{name}.{index}'
        elem = renderElement('input',
                             value=value,
                             name=name,
                             id=id,
                             cssClass=cssClass,
                             checked="checked",
                             type='radio',
                             extra='onclick="changeMatrix(this);"')
        return self._tdTemplate % (tdClass, id, text, elem)

    def renderItems(self, value):
        # check if we want to select first item, the previously selected item
        # or the "no value" item.
        if (value == self.context.missing_value
                and getattr(self, 'firstItem', False)
                and len(self.vocabulary) > 0):
            if self.context.required:
                # Grab the first item from the iterator:
                values = [next(iter(self.vocabulary)).value]
        elif value != self.context.missing_value:
            values = [value]
        else:
            values = []

        items = self.renderItemsWithValues(values)
        return items

    def renderValue(self, value):
        rendered_items = self.renderItems(value)
        return " ".join(rendered_items)


class Granting:

    principal = None

    principal_field = zope.schema.Choice(
        __name__='principal',
        source=PrincipalSource(),
        required=True)

    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.permissions = None
        self.roles = None
        self.principal_widget = None

    def status(self):
        setUpWidget(self, 'principal', self.principal_field, IInputWidget)
        if not self.principal_widget.hasInput():
            return ''
        try:
            principal = self.principal_widget.getInputValue()
        except MissingInputError:
            return ''

        self.principal = principal

        # Make sure we can use the principal id in a form by base64ing it
        principal_str = text_type(principal)
        principal_bytes = principal_str.encode('utf-8')
        principal_token = base64.b64encode(
            principal_bytes).strip().replace(b'=', b'_')
        if not isinstance(principal_token, str):
            principal_token = principal_token.decode('utf-8')

        roles = [role for name, role in getUtilitiesFor(IRole)]
        roles.sort(key=lambda x: x.title)
        principal_roles = IPrincipalRoleManager(self.context)

        self.roles = []
        for role in roles:
            name = principal_token + '.role.' + role.id
            field = zope.schema.Choice(__name__=name,
                                       title=role.title,
                                       vocabulary=settings_vocabulary)
            setUpWidget(self, name, field, IInputWidget,
                        principal_roles.getSetting(role.id, principal))
            self.roles.append(getattr(self, name + '_widget'))

        perms = [perm for name, perm in getUtilitiesFor(IPermission)]
        perms.sort(key=lambda x: x.title)
        principal_perms = IPrincipalPermissionManager(self.context)

        self.permissions = []
        for perm in perms:
            if perm.id == 'zope.Public':
                continue
            name = principal_token + '.permission.' + perm.id
            field = zope.schema.Choice(__name__=name,
                                       title=perm.title,
                                       vocabulary=settings_vocabulary)
            setUpWidget(self, name, field, IInputWidget,
                        principal_perms.getSetting(perm.id, principal))
            self.permissions.append(
                getattr(self, name + '_widget'))

        if 'GRANT_SUBMIT' not in self.request:
            return ''

        for role in roles:
            name = principal_token + '.role.' + role.id
            role_widget = getattr(self, name + '_widget')
            if role_widget.hasInput():
                try:
                    setting = role_widget.getInputValue()
                except MissingInputError:  # pragma: no cover
                    pass
                else:
                    # Arrgh!
                    if setting is Allow:
                        principal_roles.assignRoleToPrincipal(
                            role.id, principal)
                    elif setting is Deny:
                        principal_roles.removeRoleFromPrincipal(
                            role.id, principal)
                    else:
                        principal_roles.unsetRoleForPrincipal(
                            role.id, principal)

        for perm in perms:
            if perm.id == 'zope.Public':
                continue
            name = principal_token + '.permission.' + perm.id
            perm_widget = getattr(self, name + '_widget')
            if perm_widget.hasInput():
                try:
                    setting = perm_widget.getInputValue()
                except MissingInputError:  # pragma: no cover
                    pass
                else:
                    # Arrgh!
                    if setting is Allow:
                        principal_perms.grantPermissionToPrincipal(
                            perm.id, principal)
                    elif setting is Deny:
                        principal_perms.denyPermissionToPrincipal(
                            perm.id, principal)
                    else:
                        principal_perms.unsetPermissionForPrincipal(
                            perm.id, principal)

        return _('Grants updated.')



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Pluggable Authentication Service Tests

"""

__docformat__ = "reStructuredText"

import doctest
import re
import unittest

import transaction
from webtest import TestApp
from zope.app.wsgi.testlayer import encodeMultipartFormdata
from zope.app.wsgi.testlayer import http
from zope.exceptions.interfaces import UserError
from zope.interface import directlyProvides
from zope.pluggableauth.factories import Principal
from zope.testing import renormalizing

from zope.app.authentication.principalfolder import IInternalPrincipal
from zope.app.authentication.principalfolder import PrincipalFolder
from zope.app.authentication.testing import AppAuthenticationLayer


class FunkTest(unittest.TestCase):

    layer = AppAuthenticationLayer

    def setUp(self):
        super().setUp()
        self._testapp = TestApp(self.layer.make_wsgi_app())

    def publish(self, path, basic=None, form=None, headers=None):
        assert basic
        assert form
        self._testapp.authorization = ('Basic', tuple(basic.split(':')))

        env = {'wsgi.handleErrors': False}
        response = self._testapp.post(path, params=form,
                                      extra_environ=env, headers=headers)
        return response

    def getRootFolder(self):
        return self.layer.getRootFolder()

    def test_copypaste_duplicated_id_object(self):

        root = self.getRootFolder()

        # Create a principal Folder
        root['pf'] = PrincipalFolder()
        pf = root['pf']

        # Create a principal with p1 as login
        principal = Principal('p1')
        principal.login = 'p1'
        directlyProvides(principal, IInternalPrincipal)

        pf['p1'] = principal

        transaction.commit()
        self.assertEqual(len(pf.keys()), 1)

        response = self.publish('/pf/@@contents.html',
                                basic='mgr:mgrpw',
                                form={'ids:list': ['p1'],
                                      'container_copy_button': 'Copy'})
        self.assertEqual(response.status_int, 302)

        # Try to paste the file
        with self.assertRaises(UserError) as r:
            self.publish('/pf/@@contents.html',
                         basic='mgr:mgrpw',
                         form={'container_paste_button': ''})

        e = r.exception
        self.assertIn("The given name(s)", str(e))
        self.assertIn("p1", str(e))
        self.assertIn("are already being used", str(e))

    def test_cutpaste_duplicated_id_object(self):

        root = self.getRootFolder()

        # Create a principal Folder
        root['pf'] = PrincipalFolder()
        pf = root['pf']

        # Create a principal with p1 as login
        principal = Principal('p1')
        principal.login = 'p1'
        directlyProvides(principal, IInternalPrincipal)

        pf['p1'] = principal

        transaction.commit()
        self.assertEqual(len(pf.keys()), 1)

        response = self.publish('/pf/@@contents.html',
                                basic='mgr:mgrpw',
                                form={'ids:list': ['p1'],
                                      'container_cut_button': 'Cut'})
        self.assertEqual(response.status_int, 302)

        # Try to paste the file
        with self.assertRaises(UserError) as r:
            self.publish('/pf/@@contents.html',
                         basic='mgr:mgrpw',
                         form={'container_paste_button': ''})

        e = r.exception
        self.assertIn("The given name(s)", str(e))
        self.assertIn("p1", str(e))
        self.assertIn("are already being used", str(e))


checker = renormalizing.RENormalizing([
    (re.compile(r"HTTP/1\.0 200 .*"), "HTTP/1.1 200 OK"),
    (re.compile(r"HTTP/1\.0 303 .*"), "HTTP/1.1 303 See Other"),
    (re.compile(r"HTTP/1\.0 401 .*"), "HTTP/1.1 401 Unauthorized"),
    (re.compile(r"u'([^']*)'"), r"'\1'"),
])


def test_suite():
    flags = doctest.NORMALIZE_WHITESPACE | doctest.ELLIPSIS

    def _http(query_str, *args, **kwargs):
        wsgi_app = AppAuthenticationLayer.make_wsgi_app()
        # Strip leading \n
        query_str = query_str.lstrip()
        kwargs.setdefault('handle_errors', True)
        if not isinstance(query_str, bytes):
            query_str = query_str.encode("utf-8")
        return http(wsgi_app, query_str, *args, **kwargs)

    def make_doctest(path):
        test = doctest.DocFileSuite(
            path,
            checker=checker,
            optionflags=flags,
            globs={
                'http': _http,
                'encodeMultipartFormdata': encodeMultipartFormdata,
                'getRootFolder': AppAuthenticationLayer.getRootFolder
            })
        test.layer = AppAuthenticationLayer
        return test

    principalfolder = make_doctest('../principalfolder.rst')
    groupfolder = make_doctest('../groupfolder.rst')
    pau_prefix_and_searching = make_doctest('../pau_prefix_and_searching.rst')
    group_searching_with_empty_string = make_doctest(
        '../group_searching_with_empty_string.rst')
    special_groups = make_doctest('../special-groups.rst')
    issue663 = make_doctest('../issue663.rst')

    return unittest.TestSuite((
        principalfolder,
        groupfolder,
        pau_prefix_and_searching,
        group_searching_with_empty_string,
        special_groups,
        unittest.defaultTestLoader.loadTestsFromTestCase(FunkTest),
        issue663,
        doctest.DocFileSuite('../schemasearch.rst'),
    ))



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
# This package is developed by the Zope Toolkit project, documented here:
# https://zopetoolkit.readthedocs.io/
# When developing and releasing this package, please follow the documented
# Zope Toolkit policies as described by this documentation.
##############################################################################
"""Setup for zope.app.authentication package."""
import os

from setuptools import setup


def read(*rnames):
    with open(os.path.join(os.path.dirname(__file__), *rnames)) as f:
        return f.read()


tests_require = [
    'zope.app.security >= 4.0',
    'zope.app.rotterdam >= 4.0',
    'zope.app.basicskin >= 4.0',
    'zope.app.form >= 5.0',
    'zope.app.wsgi[testlayer] >= 5.3',
    'zope.app.schema',

    'zope.formlib',
    'zope.login',
    'zope.principalannotation',
    'zope.publisher >= 4.3.1',
    'zope.publisher',
    'zope.securitypolicy',
    'zope.session',
    'zope.site',
    'zope.testbrowser >= 5.2',
    'zope.testing',
    'zope.testrunner >= 6.4'
]

setup(name='zope.app.authentication',
      version='6.0',
      author='Zope Foundation and Contributors',
      author_email='zope-dev@zope.dev',
      description=('Principals and groups management for '
                   'the pluggable authentication utility'),
      long_description=(
          read('README.rst')
          + '\n\n.. contents::\n\n' +
          read('src', 'zope', 'app', 'authentication', 'README.rst')
          + '\n\n' +
          read('src', 'zope', 'app', 'authentication', 'principalfolder.rst')
          + '\n\n' +
          read('src', 'zope', 'app', 'authentication', 'vocabulary.rst')
          + '\n\n' +
          read('CHANGES.rst')
      ),
      url='http://github.com/zopefoundation/zope.app.authentication',
      license='ZPL-2.1',
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: Web Environment',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: Zope Public License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.9',
          'Programming Language :: Python :: 3.10',
          'Programming Language :: Python :: 3.11',
          'Programming Language :: Python :: 3.12',
          'Programming Language :: Python :: 3.13',
          'Programming Language :: Python :: Implementation',
          'Programming Language :: Python :: Implementation :: CPython',
          'Programming Language :: Python :: Implementation :: PyPy',
          'Natural Language :: English',
          'Operating System :: OS Independent',
          'Topic :: Internet :: WWW/HTTP',
          'Framework :: Zope :: 3',
      ],
      keywords='zope3 authentication pluggable principal group',
      python_requires='>=3.9',
      extras_require={
          'test': tests_require,
      },
      install_requires=[
          'setuptools',
          'zope.authentication',
          'zope.component',
          'zope.container',
          'zope.dublincore',
          'zope.event',
          'zope.exceptions',
          'zope.formlib >= 4.0.2',
          'zope.i18n',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.location',
          'zope.password >= 3.5.1',
          'zope.pluggableauth >= 2.2.0',
          'zope.schema',
          'zope.security',
          'zope.traversing',
          # Needed for browser code.
          'zope.app.container >= 4.0.0',
          'zope.app.component >= 4.0.0',
      ],
      include_package_data=True,
      zip_safe=False,
      )



##############################################################################
#
# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Role-Permission View Tests

"""

import unittest

import zope.interface
from zope.component.testing import PlacelessSetup as PlacefulSetup
from zope.exceptions.interfaces import UserError
from zope.i18n.interfaces import ITranslationDomain
from zope.i18nmessageid import Message
from zope.publisher.browser import BrowserView
from zope.publisher.browser import TestRequest
from zope.security.interfaces import IPermission
from zope.security.permission import Permission
from zope.securitypolicy.interfaces import IRole
from zope.securitypolicy.role import Role

from zope.app.authentication.browser import tests as ztapi
from zope.app.authentication.browser.rolepermissionview import \
    RolePermissionView
from zope.app.authentication.browser.tests.rolepermissionmanager import \
    RolePermissionManager


class RolePermissionView(RolePermissionView, BrowserView):
    """Adding BrowserView to Utilities; this is usually done by ZCML."""


@zope.interface.implementer(ITranslationDomain)
class TranslationDomain:

    def __init__(self, **translations):
        self.translations = translations

    def translate(self, msgid, *ignored, **also_ignored):
        return self.translations.get(msgid, msgid)


def defineRole(id, title=None, description=None):
    role = Role(id, title, description)
    ztapi.provideUtility(IRole, role, name=role.id)
    return role


def definePermission(id, title=None, description=None):
    permission = Permission(id, title, description)
    ztapi.provideUtility(IPermission, permission, name=permission.id)
    return permission


class FakeSiteManager:

    def __init__(self, site):
        self.__parent__ = site


class Test(PlacefulSetup, unittest.TestCase):

    def setUp(self):
        PlacefulSetup.setUp(self)
        defineRole('manager', Message('Manager', 'testdomain'))
        defineRole('member', Message('Member', 'testdomain'))
        definePermission('read', Message('Read', 'testdomain'))
        definePermission('write', Message('Write', 'testdomain'))
        site = RolePermissionManager()
        self.view = RolePermissionView(FakeSiteManager(site), None)
        ztapi.provideUtility(ITranslationDomain,
                             TranslationDomain(Member="A Member",
                                               Write="A Write",
                                               ),
                             'testdomain')

    def testRoles(self):
        self.assertEqual([role.title for role in self.view.roles()],
                         ["Member", "Manager"])

    def testPermisssions(self):
        self.assertEqual([role.title for role in self.view.permissions()],
                         ["Write", "Read"])

    def testMatrix(self):
        roles = self.view.roles()
        permissions = self.view.permissions()

        #         manager  member
        # read       +
        # write      .       -
        env = {
            'p0': 'read', 'p1': 'write',
            'r0': 'manager', 'r1': 'member',
            'p0r0': 'Allow',
            'p1r0': 'Unset', 'p1r1': 'Deny',
            'SUBMIT': 1
        }
        self.view.request = TestRequest(environ=env)
        self.view.update()

        def check(allow=('manager', 'read'), deny=('member', 'write')):
            permissionRoles = self.view.permissionRoles()
            for ip, permissionRole in enumerate(permissionRoles):
                rset = permissionRole.roleSettings()
                for ir, setting in enumerate(rset):
                    r = roles[ir].id
                    p = permissions[ip].id
                    if setting == 'Allow':
                        self.assertEqual(r, allow[0])
                        self.assertEqual(p, allow[1])
                    elif setting == 'Deny':
                        self.assertEqual(r, deny[0])
                        self.assertEqual(p, deny[1])
                    else:
                        self.assertEqual(setting, 'Unset')
        check()

        #         manager  member
        # read       -
        # write      +
        env = {
            'p0': 'read', 'p1': 'write',
            'r0': 'manager', 'r1': 'member',
            'p0r0': 'Deny',
            'p1r0': 'Allow', 'p1r1': 'Unset',
            'SUBMIT': 1
        }
        self.view.request = TestRequest(environ=env)
        self.view.update()
        check(('manager', 'write'), ('manager', 'read'))

    def testPermissionRoles(self):
        env = {'permission_id': 'write',
               'settings': ['Allow', 'Unset'],
               'SUBMIT_PERMS': 1}
        self.view.request = TestRequest(environ=env)
        self.view.update()
        permission = self.view.permissionForID('write')

        self.assertEqual('write', permission.id)
        self.assertEqual("Write", permission.title)
        self.assertIsNone(permission.description)
        settings = permission.roleSettings()
        self.assertEqual(settings, ['Allow', 'Unset'])

        env = {'permission_id': 'write',
               'settings': ['Unset', 'Deny'],
               'SUBMIT_PERMS': 1}
        self.view.request = TestRequest(environ=env)
        self.view.update()
        permission = self.view.permissionForID('write')
        settings = permission.roleSettings()
        self.assertEqual(settings, ['Unset', 'Deny'])

        env = {'permission_id': 'write',
               'settings': ['Unset', 'foo'],
               'SUBMIT_PERMS': 1}
        self.view.request = TestRequest(environ=env)
        self.assertRaises(ValueError, self.view.update)

    def testRolePermissions(self):
        env = {'Allow': ['read'],
               'Deny': ['write'],
               'SUBMIT_ROLE': 1,
               'role_id': 'member'}
        self.view.request = TestRequest(environ=env)
        self.view.update(1)
        role = self.view.roleForID('member')
        self.assertEqual('member', role.id)
        self.assertEqual('Member', role.title)
        self.assertIsNone(role.description)
        pinfos = role.permissionsInfo()
        for pinfo in pinfos:
            pid = pinfo['id']
            if pid == 'read':
                self.assertEqual(pinfo['setting'], 'Allow')
            if pid == 'write':
                self.assertEqual(pinfo['setting'], 'Deny')

        env = {'Allow': [],
               'Deny': ['read'],
               'SUBMIT_ROLE': 1,
               'role_id': 'member'}
        self.view.request = TestRequest(environ=env)
        self.view.update()
        role = self.view.roleForID('member')
        pinfos = role.permissionsInfo()
        for pinfo in pinfos:
            pid = pinfo['id']
            if pid == 'read':
                self.assertEqual(pinfo['setting'], 'Deny')
            if pid == 'write':
                self.assertEqual(pinfo['setting'], 'Unset')

    def testRolePermissions_UserError(self):
        env = {'Allow': ['read'],
               'Deny': ['read'],
               'SUBMIT_ROLE': 1,
               'role_id': 'member'}
        self.view.request = TestRequest(environ=env)
        self.assertRaises(UserError, self.view.update, 1)


def test_suite():
    return unittest.defaultTestLoader.loadTestsFromName(__name__)


if __name__ == '__main__':
    unittest.TextTestRunner().run(test_suite())



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""zope.app.authentication common test related classes/functions/objects.

"""
import zope.component
import zope.interface
from zope.app.wsgi.testlayer import BrowserLayer
from zope.testbrowser.wsgi import TestBrowserLayer

import zope.app.authentication


class _AppAuthenticationBrowserLayer(TestBrowserLayer,
                                     BrowserLayer):
    pass


AppAuthenticationLayer = _AppAuthenticationBrowserLayer(
    zope.app.authentication, allowTearDown=True)
